import 'dart:convert';
import 'dart:io';

import 'package:example/action.dart';
import 'package:example/update.dart';
import 'package:keen/client.dart';

Future<void> main(List<String> args) async {
  final email = args[0];
  print('Welcome $email!');
  final client = KeenClient<ChatAction, ChatUpdate>(
    url: 'http://localhost:4040',
    actionSerializer: (action) => action.toJson(),
    updateDeserializer: (json) => ChatUpdate.fromJson(json),
  );

  print('Requesting a password to $email...');
  final request = await client.auth.sendOneTimePasswordByEmail(
    email: email,
    deviceName: 'Console',
  );

  print('Please type the password received by email:');
  final oneTimePassword = stdin.readLineSync(encoding: utf8).trim();

  print('Requesting a token...');
  final token = await client.auth.getToken(
    sharedSecret: request,
    oneTimePassword: oneTimePassword,
  );
  print('Token : \n${jsonEncode(token.toJson())}');

  print('Refreshing the token (only needed before it expires)...');
  final newToken = await client.auth.refreshToken(token);
  print('Token : \n${jsonEncode(newToken.toJson())}');

  print('Starting a chat session...');

  final api = await client.connect(newToken);

  // Listening for updates from server
  api.updates.listen((update) {
    if (update is ReceivedMessageUpdate) {
      print('[${update.email}] ${update.content}');
    }
  });

  // Sending lines as actions to server
  stdin
      .transform(
        utf8.decoder,
      )
      .transform(
        LineSplitter(),
      )
      .listen(
    (line) async {
      print('[ME] $line');
      await api.action(
        SendMessageAction(
          line.trim(),
        ),
      );
    },
  );
}
