import 'dart:async';
import 'dart:io';

import 'package:example/action.dart';
import 'package:example/update.dart';
import 'package:keen/server.dart';

Future<void> main(List<String> args) async {
  final server = KeenServer<ChatAction, ChatUpdate>(
    auth: KeenAuthServer(
      signKey: 'my-signing-key-for-tokens',
      // FIXME only for testing, users are not persisted!
      // You can use PostgresUserStorage
      userStorage: MemoryUserStorage(),
      emailer: (request) async {
        print(
            "Sent email to '${request.email}' with password '${request.encryptedOneTimePassword}' !");
        // TODO Send an email with the encryptedPassword (app link for exemple)
        // You can use SmtpKeenEmailer for that purpose.
      },
    ),
    api: ChatApi(),
  );

  await server.run(
    address: InternetAddress.loopbackIPv4,
    port: 4040,
  );
}

class ChatApi extends KeenAuthApiServer<ChatAction, ChatUpdate> {
  ChatApi()
      : super(
          actionDeserializer: (json) => ChatAction.fromJson(json),
          updateSerializer: (update) => update.toJson(),
        );

  @override
  FutureOr<ChatUpdate> onAction(User session, ChatAction action) {
    // We send a received message to all other connected users
    if (action is SendMessageAction) {
      for (var otherSession in sessions.where((x) => x.info.id != session.id)) {
        otherSession.update(
          ReceivedMessageUpdate(
            email: session.email,
            content: action.content,
          ),
        );
      }
    }
    return null; // You can send an update to the caller if you want to
  }
}
