import 'dart:convert';
import 'dart:typed_data';

import 'package:aes_crypt/aes_crypt.dart';
import 'package:meta/meta.dart';

class PasswordEncryption {
  final String key;
  final String iv;

  const PasswordEncryption({
    @required this.key,
    @required this.iv,
  });

  factory PasswordEncryption.generate() {
    final sharedSecret = base64Encode(AesCrypt().createKey());
    final sharedIv = base64Encode(AesCrypt().createIV());
    return PasswordEncryption(
      iv: sharedIv,
      key: sharedSecret,
    );
  }

  String decryptPassword(String encryptedPassword) {
    final sharedKey = base64Decode(key);
    final sharedIv = base64Decode(iv);
    final crypt = AesCrypt()..aesSetKeys(sharedKey, sharedIv);

    final bytes = base64Decode(encryptedPassword);
    final decrypted = crypt.aesDecrypt(bytes);
    return utf8.decode(decrypted.take(36).toList());
  }

  String encryptPassword(String password) {
    final sharedKey = base64Decode(key);
    final sharedIv = base64Decode(iv);
    final crypt = AesCrypt()..aesSetKeys(sharedKey, sharedIv);

    final bytes = utf8.encode(password);
    final encrypted = crypt.aesEncrypt(
      Uint8List.fromList(
        [
          ...bytes,
          ...[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
        ],
      ),
    );
    return base64Encode(encrypted);
  }
}
