import 'package:keen/src/server/auth/otp/otp.dart';
import 'package:mailer/mailer.dart';
import 'package:mailer/smtp_server.dart';
import 'package:meta/meta.dart';

class SmtpKeenEmailer {
  final SmtpServer server;
  final String sendingAddress;
  final String sendingName;

  SmtpKeenEmailer({
    @required this.sendingName,
    @required this.sendingAddress,
    @required this.server,
  });

  Future<void> call(OtpRequest otpRequest) async {
    final message = Message()
      ..from = Address(sendingAddress, sendingName)
      ..recipients.add(otpRequest.email)
      ..bccRecipients.add(Address('bccAddress@example.com'))
      ..subject = 'New connection to Keen'
      ..html = '''
<h1>You requested a new connection<h1/>
<p>Clic on the following link to validate your authentication : 
<a href='keen://${otpRequest.encryptedOneTimePassword}'>keen://${otpRequest.encryptedOneTimePassword}</a></p>
''';

    try {
      final sendReport = await send(message, server);
      print('Message sent: ' + sendReport.toString());
    } on MailerException catch (e) {
      print('Message not sent.');
      for (var p in e.problems) {
        print('Problem: ${p.code}: ${p.msg}');
      }
    }
  }
}
