import 'package:freezed_annotation/freezed_annotation.dart';
import 'package:meta/meta.dart';

part 'user.freezed.dart';
part 'user.g.dart';

/// A unique user with [email], that has a unique internal [id].
///
/// The user is created in the database at [creationDate] after a successful authentication.
///
/// His [refreshTokens] allow him to refresh his current token.
@freezed
abstract class User with _$User {
  const factory User({
    /// The internal identifier of the user.
    @required String id,

    /// The email of the user.
    @required String email,

    /// The date of creation into the database.
    @required DateTime creationDate,

    /// All refresh tokens associated to the user.
    @required List<String> refreshTokens,
  }) = _User;

  /// Create a user from a [json] map.
  factory User.fromJson(Map<String, dynamic> json) => _$UserFromJson(json);
}
