import 'dart:async';
import 'dart:io';

import 'package:meta/meta.dart';

import 'auth/api.dart';
import 'auth/auth.dart';

class KeenServer<Action, Update> {
  KeenServer({
    @required this.auth,
    @required this.api,
    List<KeenServerController> controllers = const <KeenServerController>[],
  }) : controllers = [
          ...controllers,
          auth,
          api,
        ] {
    api.auth = auth;
  }

  final KeenAuthServer auth;
  final KeenAuthApiServer<Action, Update> api;
  final List<KeenServerController> controllers;

  Future<void> run({
    @required dynamic address,
    @required int port,
    int backlog = 0,
    bool v6Only = false,
    bool shared = false,
  }) async {
    final server = await HttpServer.bind(
      address,
      port,
      backlog: backlog,
      v6Only: v6Only,
      shared: shared,
    );
    print('[Keen] Listening on "${address}:${server.port}"');

    await for (HttpRequest request in server) {
      try {
        var processed = false;
        for (var controller in controllers) {
          if (await controller.process(request)) {
            processed = true;
            break;
          }
        }
        if (!processed) {
          print(
              '[Keen][NOT_FOUND] ${request.method} ${request.uri} from ${request.connectionInfo.remoteAddress}');
          request.response.statusCode = HttpStatus.notFound;
          await request.response.close();
        }
      } catch (e, st) {
        print('[Keen][INTERNAL_ERROR] $e');
        print('                       $st');
        request.response.statusCode = HttpStatus.internalServerError;
        await request.response.close();
      }
    }
  }
}

abstract class KeenServerController {
  FutureOr<bool> process(HttpRequest request);
}
